#include "PROGRAM\\Reports.cpp"

#define REP102_ST_OBJINIT 0
#define REP102_ST_REPORT 1
#define REP102_ST_EXPORT 2
#define REP102_ST_OBJCOMPLETE 3

//     "Rep102_Params.fr3"
//     
TDateTime rep102_sdt; //  /
TDateTime rep102_edt; //  /
Variant rep102_objArray; //    
bool rep102_needReport; //  
bool rep102_needExport; //    

//  
string rep102_expDir;

TADOConnection rep102_cnExp;
TADOQuery rep102_qryPrepData;
TADOQuery rep102_qryObjects;
TADOQuery rep102_qryReport;
TADOQuery rep102_qryExpDevSrc;
TADOQuery rep102_qryExpDevDst;
TADOQuery rep102_qryExpDataSrc;
TADOQuery rep102_qryExpDataDst;

TfrxDBDataset rep102_dsObjects;
TfrxDBDataset rep102_dsReport;

string rep102_qryPrepDataSQL1;
string rep102_qryPrepDataSQL2;
string rep102_qryObjectsSQL;
string rep102_qryReportSQL;
string rep102_qryExpDevSrcSQL;
string rep102_qryExpDataSrcSQL;
string rep102_qryExpDstSQL;

void Rep102_ClearPrepared()
{
  DeleteFile(rep_prepDir + rep_repID + "\\Part*.fp3");
  DeleteFile(rep_prepDir + rep_repID + "\\Files.csv");
  DeleteFile(rep102_expDir + "*.dbf");
}

void Rep102_InitVars()
{
  rep102_cmdStart.Text = " ";
  rep102_expDir = rep_frDir + "Export\\EIRC\\";

  rep102_cnExp = nil;
  rep102_qryPrepData = nil;
  rep102_qryObjects = nil;
  rep102_qryReport = nil;
  rep102_qryExpDevSrc = nil;
  rep102_qryExpDevDst = nil;
  rep102_qryExpDataSrc = nil;
  rep102_qryExpDataDst = nil;

  rep102_dsObjects = nil;
  rep102_dsReport = nil;

  rep102_qryPrepDataSQL1 = "SELECT rep102_init('%s', '%s', %s, %s)";
  rep102_qryPrepDataSQL2 = "SELECT rep102_prep_object(:objid)";
  rep102_qryObjectsSQL = "SELECT * FROM tmp_rep102_objects WHERE objid = :objid";
  rep102_qryReportSQL = "SELECT * FROM tmp_rep102_report WHERE objid = :objid ORDER BY vdevnum";

  rep102_qryExpDevSrcSQL =
    "SELECT datarep, numpu, type_pu, streetid, house1, house2, house3, flatnum, datastart " +
    "FROM tmp_rep102_exp_dev " +
    "WHERE vdevnum IN (SELECT vdevnum FROM tmp_rep102_vdevices WHERE objid = :objid) " +
    "ORDER BY vdevnum";
  rep102_qryExpDataSrcSQL =
    "SELECT num_pu, date_pok, prizn_nach, pokaz_pu, time_pu " +
    "FROM tmp_rep102_exp_data " +
    "WHERE vdevnum IN (SELECT vdevnum FROM tmp_rep102_vdevices WHERE objid = :objid)" +
    "ORDER BY vdevnum";
  rep102_qryExpDstSQL = "SELECT * FROM %s WHERE FALSE";
}

void Rep102_InitTasks()
{
  int i = 0; rep_tasks[i] = REP102_ST_OBJINIT;
  if (rep102_needReport) {i++; rep_tasks[i] = REP102_ST_REPORT;}
  if (rep102_needExport) {i++; rep_tasks[i] = REP102_ST_EXPORT;}
  i++; rep_tasks[i] = REP102_ST_OBJCOMPLETE;
}

void Rep102_InitCommonObjects()
{
  //     
  rep102_qryPrepData = new TADOQuery(nil);
  rep102_qryPrepData.Connection = SystemADOConnection;
  rep102_qryPrepData.LockType = 1;
  rep102_qryPrepData.SQL.Text = Format(rep102_qryPrepDataSQL1,
    [
      FormatDateTime("yyyy-mm-dd hh:nn:ss", rep102_sdt),
      FormatDateTime("yyyy-mm-dd hh:nn:ss", rep102_edt),
      Rep_GetBoolStr(rep102_needReport),
      Rep_GetBoolStr(rep102_needExport)
    ]);

  //   
  rep102_qryPrepData.Open();
  rep102_qryPrepData.Close();

  rep102_qryPrepData.SQL.Text = rep102_qryPrepDataSQL2;
  rep102_qryPrepData.Parameters.Items[0].DataType = ftInteger;

  //  
  rep102_qryObjects = new TADOQuery(nil);
  rep102_qryObjects.Connection = SystemADOConnection;
  rep102_qryObjects.SQL.Text = rep102_qryObjectsSQL;
  rep102_qryObjects.Parameters.Items[0].DataType = ftInteger;

  rep102_dsObjects = new TfrxDBDataset(nil);
  rep102_dsObjects.DataSet = rep102_qryObjects;
  rep102_dsObjects.Name = "dsObjects";
  rep102_dsObjects.CloseDataSource = true;
}

void Rep102_UninitCommonObjects()
{
  if (rep102_qryPrepData != nil) {delete rep102_qryPrepData; rep102_qryPrepData = nil;}
  if (rep102_qryObjects != nil) {delete rep102_qryObjects; rep102_qryObjects = nil;}
  if (rep102_dsObjects != nil) {delete rep102_dsObjects; rep102_dsObjects = nil;}
}

void Rep102_InitReportObjects()
{
  //       
  rep102_qryReport = new TADOQuery(nil);
  rep102_qryReport.Connection = SystemADOConnection;
  rep102_qryReport.SQL.Text = rep102_qryReportSQL;
  rep102_qryReport.Parameters.Items[0].DataType = ftInteger;

  rep102_dsReport = new TfrxDBDataset(nil);
  rep102_dsReport.DataSet = rep102_qryReport;
  rep102_dsReport.Name = "dsReport";
  rep102_dsReport.CloseDataSource = true;
}

void Rep102_UninitReportObjects()
{
  if (rep102_qryReport != nil) {delete rep102_qryReport; rep102_qryReport = nil;}
  if (rep102_dsReport != nil) {delete rep102_dsReport; rep102_dsReport = nil;}
}

void Rep102_InitExportObjects()
{
  if (!rep102_needExport) return;

  string tmplDir = rep_frDir + "DBF\\";
  string devFileName = "fl_dev";
  string dataFileName = "fl_data";

  if (!FileCopy(tmplDir + "tmpl_exp_flat_dev.dbf", rep102_expDir + devFileName + ".dbf"))
    RaiseException("    FileCopy()");
  if (!FileCopy(tmplDir + "tmpl_exp_flat_data.dbf", rep102_expDir + dataFileName + ".dbf"))
    RaiseException("    FileCopy()");

  rep102_cnExp = new TADOConnection(nil);
  rep102_cnExp.ConnectionString =
    "Provider=Microsoft.Jet.OLEDB.4.0;" +
    "Data Source=" + rep102_expDir + ";" +
    "Extended Properties=dBASE III;";
  rep102_cnExp.LoginPrompt = false;
  rep102_cnExp.Connected = true;

  rep102_qryExpDevSrc = new TADOQuery(nil);
  rep102_qryExpDevSrc.Connection = SystemADOConnection;
  rep102_qryExpDevSrc.SQL.Text = rep102_qryExpDevSrcSQL;
  rep102_qryExpDevSrc.Parameters.Items[0].DataType = ftInteger;

  rep102_qryExpDevDst = new TADOQuery(nil);
  rep102_qryExpDevDst.Connection = rep102_cnExp;
  rep102_qryExpDevDst.SQL.Text = Format(rep102_qryExpDstSQL, [devFileName]);

  rep102_qryExpDataSrc = new TADOQuery(nil);
  rep102_qryExpDataSrc.Connection = SystemADOConnection;
  rep102_qryExpDataSrc.SQL.Text = rep102_qryExpDataSrcSQL;
  rep102_qryExpDataSrc.Parameters.Items[0].DataType = ftInteger;

  rep102_qryExpDataDst = new TADOQuery(nil);
  rep102_qryExpDataDst.Connection = rep102_cnExp;
  rep102_qryExpDataDst.SQL.Text = Format(rep102_qryExpDstSQL, [dataFileName]);
}

void Rep102_UninitExportObjects()
{
  if (!rep102_needExport) return;

  if (rep102_qryExpDevSrc != nil) {delete rep102_qryExpDevSrc; rep102_qryExpDevSrc = nil;}
  if (rep102_qryExpDevDst != nil) {delete rep102_qryExpDevDst; rep102_qryExpDevDst = nil;}
  if (rep102_qryExpDataSrc != nil) {delete rep102_qryExpDataSrc; rep102_qryExpDataSrc = nil;}
  if (rep102_qryExpDataDst != nil) {delete rep102_qryExpDataDst; rep102_qryExpDataDst = nil;}
  if (rep102_cnExp != nil) {delete rep102_cnExp; rep102_cnExp = nil;}
}

void Rep102_InitTemplate()
{
  //     
  SystemReport.DataSets.Add(rep102_dsObjects);
  SystemReport.DataSets.Add(rep102_dsReport);
  //     
  TfrxMasterData b = TfrxMasterData(SystemReport.FindObject("bandReport"));
  if (b != nil) b.DataSet = rep102_dsReport;
}

//    
void Rep102_Init()
{
  Rep_InitProgress(Length(rep102_objArray), Rep102_cmdStart);
  Rep_DisplayProgress();

  Rep102_ClearPrepared();
  if (rep102_needReport) Rep_InitPrepared();

  Rep102_InitTasks();

  Rep102_InitCommonObjects();
  Rep102_InitReportObjects();
  Rep102_InitExportObjects();
  Rep102_InitTemplate();
}

//    
void Rep102_Uninit()
{
  Rep_UninitProgress(Rep102_cmdStart);
  if (rep102_needReport) Rep_UninitPrepared();

  Rep102_UninitCommonObjects();
  Rep102_UninitReportObjects();
  Rep102_UninitExportObjects();

  Rep_Uninit();
}

//    DBF  
void Rep102_ExportToDBF(int objID)
{
  try
  {
    try
    {
      rep102_cnExp.BeginTrans();

      rep102_qryExpDevSrc.Parameters.Items[0].Value = objID;
      rep102_qryExpDevSrc.Open();
      rep102_qryExpDevDst.Open();

      rep102_qryExpDataSrc.Parameters.Items[0].Value = objID;
      rep102_qryExpDataSrc.Open();
      rep102_qryExpDataDst.Open();
      int i;

      for (; !rep102_qryExpDevSrc.EOF; rep102_qryExpDevSrc.Next())
      {
        rep102_qryExpDevDst.Append();

        for (i = 0; i < rep102_qryExpDevSrc.FieldCount; i++)
          rep102_qryExpDevDst.Fields.Fields[i].Value =
            rep102_qryExpDevSrc.Fields.Fields[i].Value;

        rep102_qryExpDevDst.Post();
        Application.ProcessMessages();
        Sleep(0);
      }

      for (; !rep102_qryExpDataSrc.EOF; rep102_qryExpDataSrc.Next())
      {
        rep102_qryExpDataDst.Append();

        for (i = 0; i < rep102_qryExpDataSrc.FieldCount; i++)
          rep102_qryExpDataDst.Fields.Fields[i].Value =
            rep102_qryExpDataSrc.Fields.Fields[i].Value;

        rep102_qryExpDataDst.Post();
        Application.ProcessMessages();
        Sleep(0);
      }

      rep102_cnExp.CommitTrans();
    }
    except
    {
      Rep_ReportException();
      rep102_cnExp.RollbackTrans();
    }
  }
  finally
  {
    rep102_qryExpDevSrc.Close();
    rep102_qryExpDevDst.Close();
    rep102_qryExpDataSrc.Close();
    rep102_qryExpDataDst.Close();
  }
}

//    
void Rep102_PrepareNext()
{
  int objID;
  string fullAddr;
  string objName;
  TfrxMasterData b;

  while (true)
    switch (rep_task)
    {
      case REP102_ST_OBJINIT:
      {
        ProcessMessages();

        if (rep_break)
          Rep_ReportInfo(REP_MSG_USERBREAK);

        if ((rep_objCount >= rep_totalObjs) || rep_break)
        {
          Rep_SetTask(REP_ST_COMPLETE);
          continue;
        }

        objID = rep102_objArray[rep_objCount];
        rep102_qryPrepData.Parameters.Items[0].Value = objID;

        try
        {
          rep102_qryPrepData.Open();
        }
        except
        {
          Rep_ReportException();
          Rep_SetTask(REP102_ST_OBJCOMPLETE);
          continue;
        }

        rep102_qryObjects.Parameters.Items[0].Value = objID;
        rep102_qryObjects.Open();

        fullAddr = rep102_qryObjects.FieldByName("addr").AsString;
        objName = rep102_qryObjects.FieldByName("objname").AsString;
        if (Length(objName) > 0) fullAddr = fullAddr + " " + objName;
        Rep_WriteLog(fullAddr);

        if (rep102_needReport && rep_isSaved)
          rep_firstObjName = fullAddr;

        Rep_SetNextTask();
        continue;
      }

      case REP102_ST_REPORT:
      {
        rep102_qryReport.Parameters.Items[0].Value = objID;

        //      () 
        Rep_AsyncPrepare(rep_isSaved, "Rep102_OnComplete");
        return;
      }

      case REP102_ST_EXPORT:
      {
        Rep102_ExportToDBF(objID);
        Rep_SetNextTask();
        continue;
      }

      case REP102_ST_OBJCOMPLETE:
      {
        rep102_qryObjects.Close();
        rep102_qryPrepData.Close();
        rep_objCount++;

        if (rep102_needReport && (rep_objCount % rep_objsPerFile == 0))
          Rep_SavePrepared();

        Rep_DisplayProgress();
        Rep_SetFirstTask();
        continue;
      }

      case REP_ST_COMPLETE:
      {
        if (rep102_needReport && !rep_isSaved) Rep_SavePrepared();
        Rep102_Uninit();
        //   
        if (rep102_needReport) Rep_SelectPrepared("Rep102");
        if (rep102_needExport)
          ShellExecute(GetDesktopWindow(), "open", rep102_expDir, "", "", SW_SHOWNORMAL);
        return;
      }
    }
}

//     (RetVal- )
void Rep102_OnComplete(int RetVal)
{
  try
  {
    Rep_ProcessRetVal(RetVal);
    Rep_SetNextTask();
    Rep102_PrepareNext();
  }
  except
  {
    Rep_ReportException();
    Rep_SetTask(REP_ST_COMPLETE);
    Rep102_PrepareNext();
  }
}

//    
void Rep102_Start()
{
  try
  {
    if (!Rep_Init("Rep102")) return;
    //      
    Rep_LoadTemplate(rep_tmplDir + "Rep102_Params.fr3");
    frPrepare(1);

    //    ""  
    if (!rep_dlgResult)
    {
      Rep_Uninit();
      return;
    }

    //   
    if (!Rep_LoadTemplate(rep_tmplDir + "Rep102.fr3"))
    {
      Rep_Uninit();
      return;
    }

    Rep102_Init();
    Rep_SetFirstTask();
    Rep102_PrepareNext();
  }
  except
  {
    Rep_ReportException();
    Rep102_Uninit();
  }
}

// -------------------
//  
// -------------------

// /   
void Rep102_cmdStartOnClick(TMonText Sender)
{
  Rep102_Start();
}

//     
void Rep102_cmdSelPreparedOnClick(TMonText Sender)
{
  Rep_SelectPrepared("Rep102");
}

//     main()
{
}
