#include "PROGRAM\\Reports.cpp"

#define REP201_ST_OBJINIT 0
#define REP201_ST_REPORT 1
#define REP201_ST_OBJCOMPLETE 2

//     "Rep201_Params.fr3"
//     
TDateTime rep201_sdt; //  /
TDateTime rep201_edt; //  /
Variant rep201_objArray; //    

//  
TADOQuery rep201_qryPrepData;
TADOQuery rep201_qryContracts;
TADOQuery rep201_qryAbonents;
TADOQuery rep201_qryDevTar;

TfrxDBDataset rep201_dsContracts;
TfrxDBDataset rep201_dsAbonents;
TfrxDBDataset rep201_dsDevTar;

string rep201_qryPrepDataSQL1;
string rep201_qryPrepDataSQL2;
string rep201_qryContractsSQL;
string rep201_qryAbonentsSQL;
string rep201_qryDevTarSQL;

void Rep201_ClearPrepared()
{
  DeleteFile(rep_prepDir + rep_repID + "\\Part*.fp3");
  DeleteFile(rep_prepDir + rep_repID + "\\Files.csv");
}

void Rep201_InitVars()
{
  rep201_cmdStart.Text = "";

  rep201_qryPrepData = nil;
  rep201_qryContracts = nil;
  rep201_qryAbonents = nil;
  rep201_qryDevTar = nil;

  rep201_dsContracts = nil;
  rep201_dsAbonents = nil;
  rep201_dsDevTar = nil;

  rep201_qryPrepDataSQL1 = "SELECT rep201_init('%s', '%s')";
  rep201_qryPrepDataSQL2 = "SELECT rep201_prep_contract(:contractid)";
  rep201_qryContractsSQL = "SELECT * FROM tmp_rep201_contracts WHERE contractid = :contractid";
  rep201_qryAbonentsSQL =
    "SELECT * FROM tmp_rep201_abonents WHERE contractid = :contractid ORDER BY number, abonentid";
  rep201_qryDevTarSQL =
    "SELECT * FROM tmp_rep201_devtar WHERE contractid = :contractid ORDER BY abonnum, abonentid, sernum, id, tarnum";
}

void Rep201_InitTasks()
{
  rep_tasks[0] = REP201_ST_OBJINIT;
  rep_tasks[1] = REP201_ST_REPORT;
  rep_tasks[2] = REP201_ST_OBJCOMPLETE;
}

void Rep201_InitObjects()
{
  //     
  rep201_qryPrepData = new TADOQuery(nil);
  rep201_qryPrepData.Connection = SystemADOConnection;
  rep201_qryPrepData.LockType = 1;
  rep201_qryPrepData.SQL.Text = Format(rep201_qryPrepDataSQL1,
    [
      FormatDateTime("yyyy-mm-dd hh:nn:ss", rep201_sdt),
      FormatDateTime("yyyy-mm-dd hh:nn:ss", rep201_edt)
    ]);

  //   
  rep201_qryPrepData.Open();
  rep201_qryPrepData.Close();

  rep201_qryPrepData.SQL.Text = rep201_qryPrepDataSQL2;
  rep201_qryPrepData.Parameters.Items[0].DataType = ftInteger;

  // 
  rep201_qryContracts = new TADOQuery(nil);
  rep201_qryContracts.Connection = SystemADOConnection;
  rep201_qryContracts.SQL.Text = rep201_qryContractsSQL;
  rep201_qryContracts.Parameters.Items[0].DataType = ftInteger;

  rep201_dsContracts = new TfrxDBDataset(nil);
  rep201_dsContracts.DataSet = rep201_qryContracts;
  rep201_dsContracts.Name = "dsContracts";
  rep201_dsContracts.CloseDataSource = true;

  // 
  rep201_qryAbonents = new TADOQuery(nil);
  rep201_qryAbonents.Connection = SystemADOConnection;
  rep201_qryAbonents.SQL.Text = rep201_qryAbonentsSQL;
  rep201_qryAbonents.Parameters.Items[0].DataType = ftInteger;

  rep201_dsAbonents = new TfrxDBDataset(nil);
  rep201_dsAbonents.DataSet = rep201_qryAbonents;
  rep201_dsAbonents.Name = "dsAbonents";
  rep201_dsAbonents.CloseDataSource = true;

  //   -    
  rep201_qryDevTar = new TADOQuery(nil);
  rep201_qryDevTar.Connection = SystemADOConnection;
  rep201_qryDevTar.SQL.Text = rep201_qryDevTarSQL;
  rep201_qryDevTar.Parameters.Items[0].DataType = ftInteger;

  rep201_dsDevTar = new TfrxDBDataset(nil);
  rep201_dsDevTar.DataSet = rep201_qryDevTar;
  rep201_dsDevTar.Name = "dsDevTar";
  rep201_dsDevTar.CloseDataSource = true;
}

void Rep201_UninitObjects()
{
  if (rep201_qryPrepData != nil) {delete rep201_qryPrepData; rep201_qryPrepData = nil;}
  if (rep201_qryContracts != nil) {delete rep201_qryContracts; rep201_qryContracts = nil;}
  if (rep201_dsContracts != nil) {delete rep201_dsContracts; rep201_dsContracts = nil;}
  if (rep201_qryAbonents != nil) {delete rep201_qryAbonents; rep201_qryAbonents = nil;}
  if (rep201_dsAbonents != nil) {delete rep201_dsAbonents; rep201_dsAbonents = nil;}
  if (rep201_qryDevTar != nil) {delete rep201_qryDevTar; rep201_qryDevTar = nil;}
  if (rep201_dsDevTar != nil) {delete rep201_dsDevTar; rep201_dsDevTar = nil;}
}

void Rep201_InitTemplate()
{
  //     
  SystemReport.DataSets.Add(rep201_dsContracts);
  SystemReport.DataSets.Add(rep201_dsAbonents);
  SystemReport.DataSets.Add(rep201_dsDevTar);
  //     
  TfrxMasterData b = TfrxMasterData(SystemReport.FindObject("bandDevTar"));
  if (b != nil) b.DataSet = rep201_dsDevTar;
}

//    
void Rep201_Init()
{
  Rep_InitProgress(Length(rep201_objArray), Rep201_cmdStart);
  Rep_DisplayProgress();

  Rep201_ClearPrepared();
  Rep_InitPrepared();

  Rep201_InitTasks();
  Rep201_InitObjects();
  Rep201_InitTemplate();
}

//    
void Rep201_Uninit()
{
  Rep_UninitProgress(Rep201_cmdStart);
  Rep_UninitPrepared();
  Rep201_UninitObjects();
  Rep_Uninit();
}

//    
void Rep201_PrepareNext()
{
  int objID;
  string cNumber;
  TfrxMasterData b;

  while (true)
    switch (rep_task)
    {
      case REP201_ST_OBJINIT:
      {
        ProcessMessages();

        if (rep_break)
          Rep_ReportInfo(REP_MSG_USERBREAK);

        if ((rep_objCount >= rep_totalObjs) || rep_break)
        {
          Rep_SetTask(REP_ST_COMPLETE);
          continue;
        }

        objID = rep201_objArray[rep_objCount];
        rep201_qryPrepData.Parameters.Items[0].Value = objID;

        try
        {
          rep201_qryPrepData.Open();
        }
        except
        {
          Rep_ReportException();
          Rep_SetTask(REP201_ST_OBJCOMPLETE);
          continue;
        }

        rep201_qryContracts.Parameters.Items[0].Value = objID;
        rep201_qryContracts.Open();

        rep201_qryAbonents.Parameters.Items[0].Value = objID;
        rep201_qryAbonents.Open();

        rep201_qryDevTar.Parameters.Items[0].Value = objID;
        rep201_qryDevTar.Open();

        cNumber = rep201_qryContracts.FieldByName("number").AsString;
        Rep_WriteLog("  " + cNumber);

        if (rep_isSaved)
          rep_firstObjName = cNumber;

        Rep_SetNextTask();
        continue;
      }

      case REP201_ST_REPORT:
      {
        //      () 
        Rep_AsyncPrepare(rep_isSaved, "Rep201_OnComplete");
        return;
      }

      case REP201_ST_OBJCOMPLETE:
      {
        rep201_qryDevTar.Close();
        rep201_qryAbonents.Close();
        rep201_qryContracts.Close();
        rep201_qryPrepData.Close();
        rep_objCount++;

        if (rep_objCount % rep_objsPerFile == 0)
          Rep_SavePrepared();

        Rep_DisplayProgress();
        Rep_SetFirstTask();
        continue;
      }

      case REP_ST_COMPLETE:
      {
        if (!rep_isSaved) Rep_SavePrepared();
        Rep201_Uninit();
        //   
        Rep_SelectPrepared("Rep201");
        return;
      }
    }
}

//     (RetVal- )
void Rep201_OnComplete(int RetVal)
{
  try
  {
    Rep_ProcessRetVal(RetVal);
    Rep_SetNextTask();
    Rep201_PrepareNext();
  }
  except
  {
    Rep_ReportException();
    Rep_SetTask(REP_ST_COMPLETE);
    Rep201_PrepareNext();
  }
}

//    
void Rep201_Start()
{
  try
  {
    if (!Rep_Init("Rep201")) return;
    //      
    Rep_LoadTemplate(rep_tmplDir + "Rep201_Params.fr3");
    frPrepare(1);

    //    ""  
    if (!rep_dlgResult)
    {
      Rep_Uninit();
      return;
    }

    //   
    if (!Rep_LoadTemplate(rep_tmplDir + "Rep201.fr3"))
    {
      Rep_Uninit();
      return;
    }

    Rep201_Init();
    Rep_SetFirstTask();
    Rep201_PrepareNext();
  }
  except
  {
    Rep_ReportException();
    Rep201_Uninit();
  }
}

// -------------------
//  
// -------------------

// /   
void Rep201_cmdStartOnClick(TMonText Sender)
{
  Rep201_Start();
}

//     
void Rep201_cmdSelPreparedOnClick(TMonText Sender)
{
  Rep_SelectPrepared("Rep201");
}

//     main()
{
}

