#include "PROGRAM\\Reports.cpp"

#define REP202_ST_OBJINIT 0
#define REP202_ST_REPORT 1
#define REP202_ST_OBJCOMPLETE 2

//     "Rep202_Params.fr3"
//     
TDateTime rep202_sdt; //  /
TDateTime rep202_edt; //  /
Variant rep202_objArray; //    

//  
TADOQuery rep202_qryPrepData;
TADOQuery rep202_qryContracts;
TADOQuery rep202_qryDevTar;

TfrxDBDataset rep202_dsContracts;
TfrxDBDataset rep202_dsDevTar;

string rep202_qryPrepDataSQL1;
string rep202_qryPrepDataSQL2;
string rep202_qryContractsSQL;
string rep202_qryDevTarSQL;

void Rep202_ClearPrepared()
{
  DeleteFile(rep_prepDir + rep_repID + "\\Part*.fp3");
  DeleteFile(rep_prepDir + rep_repID + "\\Files.csv");
}

void Rep202_InitVars()
{
  rep202_cmdStart.Text = " ";

  rep202_qryPrepData = nil;
  rep202_qryContracts = nil;
  rep202_qryDevTar = nil;

  rep202_dsContracts = nil;
  rep202_dsDevTar = nil;

  rep202_qryPrepDataSQL1 = "SELECT rep202_init('%s', '%s')";
  rep202_qryPrepDataSQL2 = "SELECT rep202_prep_contract(:contractid)";
  rep202_qryContractsSQL = "SELECT * FROM tmp_rep202_contracts WHERE contractid = :contractid";
  rep202_qryDevTarSQL =
    "SELECT * FROM tmp_rep202_devtar WHERE contractid = :contractid ORDER BY get_number(abonnum), abonentid, sernum, id, tarnum";
}

void Rep202_InitTasks()
{
  rep_tasks[0] = REP202_ST_OBJINIT;
  rep_tasks[1] = REP202_ST_REPORT;
  rep_tasks[2] = REP202_ST_OBJCOMPLETE;
}

void Rep202_InitObjects()
{
  //     
  rep202_qryPrepData = new TADOQuery(nil);
  rep202_qryPrepData.Connection = SystemADOConnection;
  rep202_qryPrepData.LockType = 1;
  rep202_qryPrepData.SQL.Text = Format(rep202_qryPrepDataSQL1,
    [
      FormatDateTime("yyyy-mm-dd hh:nn:ss", rep202_sdt),
      FormatDateTime("yyyy-mm-dd hh:nn:ss", rep202_edt)
    ]);

  //   
  rep202_qryPrepData.Open();
  rep202_qryPrepData.Close();

  rep202_qryPrepData.SQL.Text = rep202_qryPrepDataSQL2;
  rep202_qryPrepData.Parameters.Items[0].DataType = ftInteger;

  // 
  rep202_qryContracts = new TADOQuery(nil);
  rep202_qryContracts.Connection = SystemADOConnection;
  rep202_qryContracts.SQL.Text = rep202_qryContractsSQL;
  rep202_qryContracts.Parameters.Items[0].DataType = ftInteger;

  rep202_dsContracts = new TfrxDBDataset(nil);
  rep202_dsContracts.DataSet = rep202_qryContracts;
  rep202_dsContracts.Name = "dsContracts";
  rep202_dsContracts.CloseDataSource = true;

  //   -    
  rep202_qryDevTar = new TADOQuery(nil);
  rep202_qryDevTar.Connection = SystemADOConnection;
  rep202_qryDevTar.SQL.Text = rep202_qryDevTarSQL;
  rep202_qryDevTar.Parameters.Items[0].DataType = ftInteger;

  rep202_dsDevTar = new TfrxDBDataset(nil);
  rep202_dsDevTar.DataSet = rep202_qryDevTar;
  rep202_dsDevTar.Name = "dsDevTar";
  rep202_dsDevTar.CloseDataSource = true;
}

void Rep202_UninitObjects()
{
  if (rep202_qryPrepData != nil) {delete rep202_qryPrepData; rep202_qryPrepData = nil;}
  if (rep202_qryContracts != nil) {delete rep202_qryContracts; rep202_qryContracts = nil;}
  if (rep202_dsContracts != nil) {delete rep202_dsContracts; rep202_dsContracts = nil;}
  if (rep202_qryDevTar != nil) {delete rep202_qryDevTar; rep202_qryDevTar = nil;}
  if (rep202_dsDevTar != nil) {delete rep202_dsDevTar; rep202_dsDevTar = nil;}
}

void Rep202_InitTemplate()
{
  //     
  SystemReport.DataSets.Add(rep202_dsContracts);
  SystemReport.DataSets.Add(rep202_dsDevTar);
  //     
  TfrxMasterData b = TfrxMasterData(SystemReport.FindObject("bandDevTar"));
  if (b != nil) b.DataSet = rep202_dsDevTar;
}

//    
void Rep202_Init()
{
  Rep_InitProgress(Length(rep202_objArray), Rep202_cmdStart);
  Rep_DisplayProgress();

  Rep202_ClearPrepared();
  Rep_InitPrepared();

  Rep202_InitTasks();
  Rep202_InitObjects();
  Rep202_InitTemplate();
}

//    
void Rep202_Uninit()
{
  Rep_UninitProgress(Rep202_cmdStart);
  Rep_UninitPrepared();
  Rep202_UninitObjects();
  Rep_Uninit();
}

//    
void Rep202_PrepareNext()
{
  int objID;
  string cNumber;
  TfrxMasterData b;

  while (true)
    switch (rep_task)
    {
      case REP202_ST_OBJINIT:
      {
        ProcessMessages();

        if (rep_break)
          Rep_ReportInfo(REP_MSG_USERBREAK);

        if ((rep_objCount >= rep_totalObjs) || rep_break)
        {
          Rep_SetTask(REP_ST_COMPLETE);
          continue;
        }

        objID = rep202_objArray[rep_objCount];
        rep202_qryPrepData.Parameters.Items[0].Value = objID;

        try
        {
          rep202_qryPrepData.Open();
        }
        except
        {
          Rep_ReportException();
          Rep_SetTask(REP202_ST_OBJCOMPLETE);
          continue;
        }

        rep202_qryContracts.Parameters.Items[0].Value = objID;
        rep202_qryContracts.Open();

        rep202_qryDevTar.Parameters.Items[0].Value = objID;
        rep202_qryDevTar.Open();

        cNumber = rep202_qryContracts.FieldByName("number").AsString;
        Rep_WriteLog("  " + cNumber);

        if (rep_isSaved)
          rep_firstObjName = cNumber;

        Rep_SetNextTask();
        continue;
      }

      case REP202_ST_REPORT:
      {
        //      () 
        Rep_AsyncPrepare(rep_isSaved, "Rep202_OnComplete");
        return;
      }

      case REP202_ST_OBJCOMPLETE:
      {
        rep202_qryDevTar.Close();
        rep202_qryContracts.Close();
        rep202_qryPrepData.Close();
        rep_objCount++;

        if (rep_objCount % rep_objsPerFile == 0)
          Rep_SavePrepared();

        Rep_DisplayProgress();
        Rep_SetFirstTask();
        continue;
      }

      case REP_ST_COMPLETE:
      {
        if (!rep_isSaved) Rep_SavePrepared();
        Rep202_Uninit();
        //   
        Rep_SelectPrepared("Rep202");
        return;
      }
    }
}

//     (RetVal- )
void Rep202_OnComplete(int RetVal)
{
  try
  {
    Rep_ProcessRetVal(RetVal);
    Rep_SetNextTask();
    Rep202_PrepareNext();
  }
  except
  {
    Rep_ReportException();
    Rep_SetTask(REP_ST_COMPLETE);
    Rep202_PrepareNext();
  }
}

//    
void Rep202_Start()
{
  try
  {
    if (!Rep_Init("Rep202")) return;
    //      
    Rep_LoadTemplate(rep_tmplDir + "Rep202_Params.fr3");
    frPrepare(1);

    //    ""  
    if (!rep_dlgResult)
    {
      Rep_Uninit();
      return;
    }

    //   
    if (!Rep_LoadTemplate(rep_tmplDir + "Rep202.fr3"))
    {
      Rep_Uninit();
      return;
    }

    Rep202_Init();
    Rep_SetFirstTask();
    Rep202_PrepareNext();
  }
  except
  {
    Rep_ReportException();
    Rep202_Uninit();
  }
}

// -------------------
//  
// -------------------

// /   
void Rep202_cmdStartOnClick(TMonText Sender)
{
  Rep202_Start();
}

//     
void Rep202_cmdSelPreparedOnClick(TMonText Sender)
{
  Rep_SelectPrepared("Rep202");
}

//     main()
{
}

