#define REP_ST_COMPLETE -1

#define REP_STR_BREAK "  "
#define REP_STR_PROGRESS " %d   %d"
#define REP_SAVE_FILE_NAME "Part%3.3d.fp3"

#define REP_MSG_USERBREAK "   "

string rep_LMWD;
string rep_frDir;
string rep_tmplDir;
string rep_prepDir;

string rep_repID;
string rep_dbConnString;
bool rep_hadConnection;

bool rep_inProgress;
bool rep_break;
string rep_strStart;
int rep_totalObjs;
int rep_objCount;

int rep_tasks[5];
int rep_taskIdx;
int rep_task;

string rep_curTmplFileName;
string rep_newTmplFileName;

string rep_logFilePath;
string rep_saveFilePath;
bool rep_isSaved;
int rep_fileCount;
TFileStream rep_infoFile;
string rep_firstObjName;

//  
int rep_objsPerFile; //     
bool rep_dlgResult; //    

// -------------------
//  
// -------------------

void Rep_InitVars()
{
  rep_LMWD = IncludeTrailingBackslash(LMWorkDir());
  rep_frDir = rep_LMWD + "FR\\";
  rep_prepDir = rep_frDir + "Prepared\\";

  rep_dbConnString = SystemADOConnection.ConnectionString;
  rep_inProgress = false;
}

bool Rep_Init(string repID)
{
  if (rep_inProgress)
  {
    // ,      
    if (rep_repID == repID)
      rep_break = true;
    else
      LMShowMessage("      ");

    return false;
  }

  rep_repID = repID;
  rep_logFilePath = rep_prepDir + rep_repID + "\\Log.txt";
  Rep_WriteLog("");
  Rep_WriteLog("   ...");

  rep_tmplDir = rep_frDir + repID + "\\";
  rep_curTmplFileName = "";
  rep_dlgResult = false;
  Rep_ConnectDB();
  return true;
}

void Rep_Uninit()
{
  rep_repID = "";
  rep_tmplDir = "";
  rep_curTmplFileName = "";
  Rep_LoadTemplate("");
  rep_dlgResult = false;
  Rep_DisconnectDB();

  Rep_WriteLog("   ");
  rep_logFilePath = "";
}

//     
void Rep_WriteLog(string s)
{
  LMProtokol(s, rep_logFilePath);
}

//        
void Rep_ReportMessage(string s, bool isError)
{
  LMProtokol(s, rep_logFilePath);

  if (isError)
    LMShowMessage2(s, "", false, true, poScreenCenter);
  else
    LMShowMessage2(s, "", false, false, poScreenCenter);
}

//     
void Rep_ReportException()
{
  Rep_ReportMessage(Format("  ( '%s'):\r\n\r\n%s",
    [ExceptionClassName, ExceptionMessage]), true);
}

//    
void Rep_ReportError(string s)
{
  Rep_ReportMessage(s, true);
}

//   
void Rep_ReportInfo(string s)
{
  Rep_ReportMessage(s, false);
}

//  SQL boolean
string Rep_GetBoolStr(bool value)
{
  if (value) return "TRUE"; else return "FALSE";
}

// ---------------
//   
// ---------------

//    
void Rep_ConnectDB()
{
  rep_hadConnection = SystemADOConnection.Connected;

  if (!rep_hadConnection)
  {
    //  Windows Vista     
    SystemADOConnection.ConnectionString = rep_dbConnString;
    SystemADOConnection.Connected = true;
  }
}

//    
void Rep_DisconnectDB()
{
  if (!rep_hadConnection)
    SystemADOConnection.Connected = false;
}

// ---------------------------------
//    
// ---------------------------------

void Rep_InitProgress(int totalObjs, TMonControl ctl)
{
  rep_inProgress = true;
  rep_break = false;
  rep_totalObjs = totalObjs;
  rep_objCount = 0;

  rep_strStart = ctl.Text;
  ctl.Text = REP_STR_BREAK;
  Rep_pbProgress.MaxValue = rep_totalObjs;
}

void Rep_UninitProgress(TMonControl ctl)
{
  ctl.Text = rep_strStart;
  Rep_pbProgress.CurValue = 0;
  Rep_pbProgress.Text = "";
  rep_inProgress = false;
}

void Rep_DisplayProgress()
{
  Rep_pbProgress.CurValue = rep_objCount;
  Rep_pbProgress.Text = Format(REP_STR_PROGRESS, [rep_objCount, rep_totalObjs]);
  Application.ProcessMessages();
}

// -------------------------------------
//    
// -------------------------------------

void Rep_SetFirstTask()
{
  rep_taskIdx = 0;
  rep_task = rep_tasks[rep_taskIdx];
}

void Rep_SetNextTask()
{
  rep_taskIdx++;
  rep_task = rep_tasks[rep_taskIdx];
}

void Rep_SetTask(int task)
{
  rep_task = task;
}

// --------------------------------------------
//     FastReport
// --------------------------------------------

bool Rep_LoadTemplate(string FileName)
{
  if (frLoad(FileName) != 0)
  {
    rep_curTmplFileName = FileName;
    return true;
  }
  else
  {
    if (FileName != "")
      Rep_ReportError("   : '" + FileName + "'");

    return false;
  }
}

void Rep_AsyncPrepare(bool clearLastReport, string onReportComplete)
{
  if (!frAsyncPrepare(clearLastReport, onReportComplete))
    RaiseException("      ");
}

void Rep_ProcessRetVal(int retVal)
{
  //   :
  // 0 -  ; 1 - ; 2 -   frBreak()
  switch (retVal)
  {
    case 0:
      Rep_ReportError("      FastReport");

    case 1:
      rep_isSaved = false;

    case 2:
      Rep_ReportInfo("   ");
  }
}

// ---------------------------------
//     
// ---------------------------------

void Rep_InitPrepared()
{
  rep_saveFilePath = rep_prepDir + rep_repID + "\\" + REP_SAVE_FILE_NAME;
  rep_isSaved = true;
  rep_fileCount = 0;
  rep_infoFile = nil;
  rep_infoFile =
    new TFileStream(rep_prepDir + rep_repID + "\\Files.csv", fmCreate);
}

void Rep_UninitPrepared()
{
  if (rep_infoFile != nil)
    delete rep_infoFile;
}

void Rep_SavePrepared()
{
  rep_fileCount++;
  string fName = Format(rep_saveFilePath, [rep_fileCount]);
  rep_isSaved = true;

  try
  {
    SystemReport.PreviewPages.SaveToFile(fName);
  }
  except
  {
    Rep_ReportError("     : '" + fName + "'");
    return;
  }

  string s = Format("%s;%s\r\n", [rep_firstObjName, fName]);

  try
  {
    rep_infoFile.Write(s, Length(s));
  }
  except
  {
    Rep_ReportError("     'Files.csv'");
  }
}

// -----------------------------------
//    
// -----------------------------------

void Rep_PrepareReports()
{
  mapReports.Show();
}

// ------------------------
//   
// ------------------------

//     
void Rep_SelectPrepared(string repID)
{
  string repInfoFileName = rep_prepDir + repID + "//Files.csv";
  string objName;

  Rep_lstPrepared.Tag2 = repInfoFileName;
  TStrings items = Rep_lstPrepared.List.Items;
  items.Clear();

  for (int i = 0; true; i++)
  {
    objName = LMGetCSV(repInfoFileName, i, 0);
    if (objName == "0") break;
    items.Add(objName + "    ...");
  }

  if (items.Count > 0)
    Rep_lstPrepared.List.ItemIndex = 0;

  mapSelPreparedRep.Show();
}

//     
void Rep_ShowPrepared()
{
  int i = Rep_lstPrepared.List.ItemIndex;
  if (i == -1) return;
  string fName = LMGetCSV(Rep_lstPrepared.Tag2, i, 1);

  if (SystemReport.PreviewPages.LoadFromFile(fName))
    frShowPrepared();
  else
    LMShowMessage("    :\r\n'" + fName + "'");
}

// -------------------
//  
// -------------------

//   "  ..."
// ----------------------------------

void Rep_cmdRepCloseOnClick(TMonText Sender)
{
  mapReports.Close();
}

//   " "
// --------------------------

void Rep_cmdShowPreparedOnClick(TMonText Sender)
{
  Rep_ShowPrepared();
}

void Rep_lstPreparedOnDblClick(TMonList Sender)
{
  Rep_ShowPrepared();
}

void Rep_cmdSelPrepCloseOnClick(TMonText Sender)
{
  mapSelPreparedRep.Close();
}

//     main()
{
}

